"""
Will Van Rensselaer (wavanrensselaer@wpi.edu)
MU 2300 - Final Project
"""

import random
import notes
import mido
from mido import MidiFile
from mido import MetaMessage
from mido import Message
from mido import MidiTrack

def create(name, tempo, key, beats, rand):
	print('name: {}, tempo: {}, key: {}, beats: {}'.format(name, tempo, key, beats))
	file = MidiFile()

	file.add_track(name)
	header = file.tracks[0]
	header.append(MetaMessage('set_tempo', tempo=tempo))
	header.append(MetaMessage('key_signature', key=key))
	header.append(MetaMessage('end_of_track'))

	file.add_track(name)
	track = file.tracks[1]
	messages = []

	if rand:
		messages = create_random_notes(key, beats)
	else:
		messages = create_notes(key, beats)

	for i in range(len(messages)):
		track.append(messages[i])

	track.append(MetaMessage('end_of_track'))

	return file

def create_random_notes(key, beats):
	messages = []

	for i in range(beats):
		note_type = random.choice([notes.SIXTEENTH, notes.EIGHTH, notes.QUARTER])

		for j in range(int(notes.BEAT / note_type)):
			note = notes.random_note(octave=random.randint(-1, 1), key=key)
			messages.append(Message('note_on', note=note, velocity=80, time=0))
			messages.append(Message('note_off', note=note, time=note_type))
	
	return messages

def create_notes(key, beats):
	messages = []
	total_ticks = beats * notes.BEAT
	ticks = 0

	note = notes.get_note_in_key(notes.get_key_note(key), key=key)

	while ticks < total_ticks:
		progression = random.choice(notes.progressions)

		for i in range(len(progression)):
			note_type = random.choice([notes.SIXTEENTH, notes.EIGHTH, notes.QUARTER])
			value = note + progression[i]

			if (ticks + note_type) > total_ticks:
				note_type = total_ticks - ticks

			messages.append(Message('note_on', note=value, velocity=80, time=0))
			messages.append(Message('note_off', note=value, time=note_type))

			ticks += note_type

			if ticks >= total_ticks:
				break

	return messages

def print_file(file):
	for i, track in enumerate(file.tracks):
		print('Track {}: {}'.format(i, track.name))
		for message in track:
			print(message)
